using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace WinFormMCalc
{
	/// <summary>
	/// Class Model.cs
	/// jlouie 07.07.02
	/// Adapted from Model.java
	/// "Visual Cafe for Java Explorer, Database Development Edition"
	/// William Brogden, Jeffrey A. Louie, and Ed Tittel, Coriolis, 1998, 585pp.
	/// Supplied "as is"
	/// No warranty is expressed or implied
	/// This code is for instructional use only
	/// </summary>
	public class Model 
	{
		// internal class constants, not "versionable"
		private const int INVALID= -1;  // flags error
		private const int PRINCIPAL= 0;
		private const int INTEREST= 1;
		private const int MONTHS= 2;
		private const int PAYMENT= 3;

		private double[] arrayDb= new double[4];
		private int target= INVALID;
		private string message= "";

		/* // uncomment to run console self test
		// self test static method outputs state to console
		static void ConsoleDebug(Model model) 
		{
			if (model == null) 
			{
				System.Console.WriteLine("Null object.");
				return;
			}
			System.Console.WriteLine("Message: "+model.Message);
			System.Console.WriteLine("Result: "+model.Result);
			System.Console.WriteLine("Principal: "+model.Principal);
			System.Console.WriteLine("Interest: "+model.Interest);
			System.Console.WriteLine("Months: "+model.Months);
			System.Console.WriteLine("Payment: "+model.Payment);
		}
		*/

		/* // uncomment to run console self test
		// self test
		[STAThread]
		static void Main() 
		{ 
			// test internal consistency of algorithms
			Model model= new Model(100000,8.5,360,0);
			Model.ConsoleDebug(model); // payment = 768.9134584334
			model.Init(0,8.5,360,768.9134584334);
			Model.ConsoleDebug(model);
			model.Init(100000,0,360,768.9134584334);
			Model.ConsoleDebug(model);
			model.Init(100000,8.5,0,768.9134584334);
			Model.ConsoleDebug(model);
			System.Console.ReadLine();
		}*/
	 
		// no arg constructor
		public Model(){;}

		// arg constructor
		public Model(double principal, double interest, int months, double payment) 
		{
			Init(principal, interest, months, payment);
		}

		// factored code, can be called after call to constructor
		// allowing reuse of instance of class
		// eg. object is _not_ immutable by design
		public void Init(double principal, double interest, int months, double payment) 
		{
			// reset flags
			target= INVALID;
			message= "";

			// store input into array of double
			arrayDb[PRINCIPAL]= principal;
			arrayDb[INTEREST]= interest;
			arrayDb[MONTHS]= (double)months;
			arrayDb[PAYMENT]= payment;

			// validate input

			// one, and only one, "value" must be zero --> target
			int zeros= 0;
			int tempTarget= INVALID;
			for (int i=0; i<4; i++) 
			{
				if (arrayDb[i] == 0) 
				{
					zeros++;
					tempTarget= i;
				}
			}
			if (zeros>1) 
			{
				message= "Too many zero parameters.";
				return;
			}
			if (zeros == 0) 
			{
				message= "One parameter must be zero.";
				return;
			}

			// validate interest
			if (interest > 100 || interest < 0) 
			{
				message= "Invalid interest.";
				return;
			}

			// validate months
			if (months < 0) 
			{
				message= "Invalid months.";
				return;
			}

			// validate principal
			if (principal < 0) 
			{
				message= "Invalid principal.";
				return;
			}

			// validate payment
			if (payment < 0) 
			{
				message= "Invalid payment.";
				return;
			}

			// input parameters appear valid
			target= tempTarget;
			DoAlgorithm(target);
		}

		// the actual amortization algorithm
		// m= P*i(1-(1+i)^-N)
		// i=r/1200
		// result= 0 --> marks error
		private void DoAlgorithm(int target) 
		{
			double result= 0;
			double P= arrayDb[PRINCIPAL];  // principal
			double i= arrayDb[INTEREST]/1200; // monthly percentage rate
			double N= arrayDb[MONTHS]; // loan period in months
			double m= arrayDb[PAYMENT]; // monthly payment
			
			if (target>= 0 && target< 4) // validate target
			{
				try 
				{
					switch (target) 
					{
						case PRINCIPAL:  // principal
							result= 1+i;
							result= 1/Math.Pow(result, N);
							result= ((1-result)/i)*m;
							break;
						case INTEREST:  // annual interest
							// algorithm fails if N*m >= P !!
							if ((N*m)<P) 
							{
								throw new ArithmeticException();
							}
							// factor out Interest function, too long
							result= CalcInterest(P,N,m);
							break;
						case MONTHS:  // loan period
							result= (1-(P*i/m));
							result= Math.Log(result);
							result= -result/Math.Log((1+i));
							break;
						case PAYMENT:  // monthly payments
							result= 1+i;
							result= 1/Math.Pow(result,N);
							result= (P*i)/(1-result);
							break;
							//default:
							//break;
					}
				}
				catch 
				{
					result= 0;
				}
			}

			// validate result
			if (Double.IsNaN(result)) 
			{
				result= 0;
			}
			if (result == 0) 
			{
				message= "Input Error.";
			}
			else // valid result
			{
				arrayDb[target]= result;
			}
		}
		
		// a complex iterative calculation for interest
		// thanks to Dr. W. Carlini (and Newton)for the solution
		// returns zero on error
		// ASSERT (N*m)>=P
		private double CalcInterest(double P, double N, double m) 
		{
			double temp= (m/P), answer= (m/P), diff= 100, numerator= 0, denominator= 0, accuracy= .00001;
			int index, maxIterations= 1000;

			try 
			{
				for (index= 0; ((diff>accuracy) && (index<maxIterations)); index++) 
				{
					temp= answer;
					numerator= (P*temp/m)+Math.Pow((1+temp),-N)-1;
					denominator= (P/m)-N*Math.Pow((1+temp),(-N-1));
					// if (denominator ==0 ){throw new ArithmeticException();}
					answer= temp-(numerator/denominator);
					diff= answer- temp;
					if (diff<0) 
					{
						diff= -diff;
					}
				}
				answer *= 1200;

				// validate answer
				if ((answer<0) || Double.IsNaN(answer) || (index == maxIterations)) 
				{
					throw new ArithmeticException();
				}
			}
			catch 
			{
				answer= 0;
			}
			return answer;
		}

		// default target is -1 (INVALID)
		public bool IsValid() 
		{
			return ((target>=PRINCIPAL) && (target<=PAYMENT)) ? true: false;
		}

		// Java "getter" code converted to C# get only properties
		public double Result 
		{
			get {
				if (IsValid()) 
				{
					return arrayDb[target];
				}
				else 
				{
					return 0.0;
				}
			}
		}
		public int Target 
		{
			get 
			{
				return target;
			}
		}
		public string Message 
		{
			get 
			{
				return message;
			}
		}
		public double Principal
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[PRINCIPAL];
				}
				else 
				{
					return 0.0;
				}
			}
		}
		public double Interest 
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[INTEREST];
				}
				else 
				{
					return 0.0;
				}
			}
		}
		public double Months
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[MONTHS];
				}
				else 
				{
					return 0;
				}
			}
		}
		public double Payment 
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[PAYMENT];
				}
				else 
				{
					return 0.0;
				}
			}
		}
	}

	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.TextBox textBoxPrincipal;
		private System.Windows.Forms.TextBox textBoxInterest;
		private System.Windows.Forms.TextBox textBoxPeriod;
		private System.Windows.Forms.TextBox textBoxPayment;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.Button buttonCalculate;
		private System.Windows.Forms.Button buttonReset;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.TextBox textBoxMessage;
		private Model model= new Model();

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.textBoxPrincipal = new System.Windows.Forms.TextBox();
			this.textBoxInterest = new System.Windows.Forms.TextBox();
			this.textBoxPeriod = new System.Windows.Forms.TextBox();
			this.textBoxPayment = new System.Windows.Forms.TextBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.textBoxMessage = new System.Windows.Forms.TextBox();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.buttonCalculate = new System.Windows.Forms.Button();
			this.buttonReset = new System.Windows.Forms.Button();
			this.label5 = new System.Windows.Forms.Label();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.SuspendLayout();
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.textBoxPayment,
																					this.textBoxPeriod,
																					this.textBoxInterest,
																					this.textBoxPrincipal,
																					this.label4,
																					this.label3,
																					this.label2,
																					this.label1});
			this.groupBox1.Location = new System.Drawing.Point(16, 16);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(288, 144);
			this.groupBox1.TabIndex = 0;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Twisted Mortgage Calculator";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(16, 32);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(104, 23);
			this.label1.TabIndex = 0;
			this.label1.Text = "Principal";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(16, 56);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(104, 23);
			this.label2.TabIndex = 1;
			this.label2.Text = "Annual Interest";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(16, 80);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(104, 23);
			this.label3.TabIndex = 2;
			this.label3.Text = "Period (Months)";
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(16, 104);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(104, 23);
			this.label4.TabIndex = 3;
			this.label4.Text = "Monthly Payment";
			// 
			// textBoxPrincipal
			// 
			this.textBoxPrincipal.Location = new System.Drawing.Point(136, 32);
			this.textBoxPrincipal.Name = "textBoxPrincipal";
			this.textBoxPrincipal.Size = new System.Drawing.Size(128, 20);
			this.textBoxPrincipal.TabIndex = 4;
			this.textBoxPrincipal.Text = "";
			// 
			// textBoxInterest
			// 
			this.textBoxInterest.Location = new System.Drawing.Point(136, 56);
			this.textBoxInterest.Name = "textBoxInterest";
			this.textBoxInterest.Size = new System.Drawing.Size(128, 20);
			this.textBoxInterest.TabIndex = 5;
			this.textBoxInterest.Text = "";
			// 
			// textBoxPeriod
			// 
			this.textBoxPeriod.Location = new System.Drawing.Point(136, 80);
			this.textBoxPeriod.Name = "textBoxPeriod";
			this.textBoxPeriod.Size = new System.Drawing.Size(128, 20);
			this.textBoxPeriod.TabIndex = 6;
			this.textBoxPeriod.Text = "";
			// 
			// textBoxPayment
			// 
			this.textBoxPayment.Location = new System.Drawing.Point(136, 104);
			this.textBoxPayment.Name = "textBoxPayment";
			this.textBoxPayment.Size = new System.Drawing.Size(128, 20);
			this.textBoxPayment.TabIndex = 7;
			this.textBoxPayment.Text = "0";
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.textBoxMessage});
			this.groupBox2.Location = new System.Drawing.Point(16, 168);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(288, 64);
			this.groupBox2.TabIndex = 1;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Error Messages";
			// 
			// textBoxMessage
			// 
			this.textBoxMessage.AutoSize = false;
			this.textBoxMessage.Enabled = false;
			this.textBoxMessage.Location = new System.Drawing.Point(8, 16);
			this.textBoxMessage.Name = "textBoxMessage";
			this.textBoxMessage.Size = new System.Drawing.Size(272, 40);
			this.textBoxMessage.TabIndex = 0;
			this.textBoxMessage.Text = "";
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.label5,
																					this.buttonReset,
																					this.buttonCalculate});
			this.groupBox3.Location = new System.Drawing.Point(16, 240);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(288, 48);
			this.groupBox3.TabIndex = 2;
			this.groupBox3.TabStop = false;
			// 
			// buttonCalculate
			// 
			this.buttonCalculate.Location = new System.Drawing.Point(120, 16);
			this.buttonCalculate.Name = "buttonCalculate";
			this.buttonCalculate.TabIndex = 0;
			this.buttonCalculate.Text = "Calculate";
			this.buttonCalculate.Click += new System.EventHandler(this.buttonCalculate_Click);
			// 
			// buttonReset
			// 
			this.buttonReset.Location = new System.Drawing.Point(208, 16);
			this.buttonReset.Name = "buttonReset";
			this.buttonReset.TabIndex = 1;
			this.buttonReset.Text = "Reset";
			this.buttonReset.Click += new System.EventHandler(this.buttonReset_Click);
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(8, 16);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(100, 16);
			this.label5.TabIndex = 2;
			this.label5.Text = "Set Target to Zero";
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(336, 310);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.groupBox3,
																		  this.groupBox2,
																		  this.groupBox1});
			this.MinimumSize = new System.Drawing.Size(344, 344);
			this.Name = "Form1";
			this.Text = "Form1";
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void buttonCalculate_Click(object sender, System.EventArgs e)
		{
			double principal= 0;
			double interest= 0;
			int months= 0;
			double payment= 0;
			bool isInputError= false;

			// validate user input, must allow zero
			try 
			{
				principal= Double.Parse(textBoxPrincipal.Text);
				if (principal<0) 
				{
					throw new Exception();
				}
			}
			catch 
			{
				textBoxPrincipal.Text= "Invalid Input.";
				isInputError= true;
			}
			try 
			{
				interest= Double.Parse(textBoxInterest.Text);
				if ((interest < 0) || (interest > 100)) 
				{
					throw new Exception();
				}
			}
			catch 
			{
				textBoxInterest.Text= "Invalid Input.";
				isInputError= true;
			}
			try 
			{
				months= Int32.Parse(textBoxPeriod.Text);
				if (months <0)
				{
					throw new Exception();
				}
			}
			catch 
			{
				textBoxPeriod.Text= "Invalid Input.";
				isInputError= true;
			}
			try 
			{
				payment= Double.Parse(textBoxPayment.Text);
				if (payment < 0) 
				{
					throw new Exception();
				}
			}
			catch 
			{
				textBoxPayment.Text= "Invalid Input.";
				isInputError= true;
			}
			if (isInputError) 
			{
				return;
			}

			// valid user input
			model.Init(principal,interest,months,payment);
			if (model.IsValid()) 
			{
				textBoxPrincipal.Text= model.Principal.ToString();
				textBoxInterest.Text= model.Interest.ToString();
				textBoxPeriod.Text= model.Months.ToString();
				textBoxPayment.Text= model.Payment.ToString();
				textBoxMessage.Text= model.Message.ToString();
			}
			else 
			{
				ResetControls();
				textBoxMessage.Text= model.Message.ToString();
			}
			
		}

		private void buttonReset_Click(object sender, System.EventArgs e)
		{
			ResetControls();
			textBoxMessage.Text= "";
		}

		private void ResetControls() 
		{
			textBoxPrincipal.Text= "";
			textBoxInterest.Text= "";
			textBoxPeriod.Text= "";
			textBoxPayment.Text= "0";			
		}
	}
}
